#!/bin/bash

IP=$(ip route get 8.8.8.8 | awk -F"src " 'NR==1{split($2,a," ");print a[1]}')
INTERFACE=$(ip route get 8.8.8.8 | awk -F"dev " 'NR==1{split($2,a," ");print a[1]}')

OLDIFS=$IFS
IFS="."
read ip1 ip2 ip3 ip4 <<< "$IP"
VIP="$ip1.$ip2.$ip3.25" # virtual ip addr
IFS=$OLDIFS


sudo swapoff -a # turn off swap memory
# put load balancer pod spec in the manifests directory
sudo docker run --network host --rm ghcr.io/kube-vip/kube-vip:main manifest pod \
--vip "$VIP" \
--interface "$INTERFACE" \
--arp \
--controlplane \
--leaderElection | sudo tee /etc/kubernetes/manifests/vip.yaml

# load up endpoint in the config file
configFile="kubeadm-config.yaml"
newIP="$VIP"
key="controlPlaneEndpoint"
sed -r "s/^(\s*${key}\s*:\s*).*/\1"${newIP}:6443"/" -i "$configFile"

# key1="apiServer"
# key2="advertiseAddress"
# sed -r "s/^(\s*${key1}\s*:\s*${key2}\s*).*/\1${newIP}/" -i "$configFile"
#
# key="apiServerCertSANs"
# sed -r "s/^(\s*${key1}\s*:\s*).*/\1${newIP}/" -i "$configFile"

sudo kubeadm init --config kubeadm-config.yaml --upload-certs

if [[ $? -ne 0 ]] ; then
  # some error
  echo "Error! Stopping cluster set up"
  exit 1
fi

mkdir -p $HOME/.kube
sudo cp -f /etc/kubernetes/admin.conf $HOME/.kube/config
sudo chown $(id -u):$(id -g) $HOME/.kube/config

kubectl apply -f flannel.yml

# create control-plane join command and copy it to clip
OLDIFS=$IFS
IFS=''
ifCMD="ip route get 8.8.8.8 | awk -F'dev ' 'NR==1{split(\$2,a,\" \");print a[1]}'"
vipCMD="sudo docker run --network host --rm ghcr.io/kube-vip/kube-vip:main manifest pod --vip "$VIP" \
--interface wlan0 \
--arp \
--controlplane \
--leaderElection | sudo tee /etc/kubernetes/manifests/vip.yaml"

# echo $(/bin/bash -c "$ifCMD")
# echo $(/bin/bash -c "$vipCMD")
IFS=$OLDIFS


echo "------------------------------------------"
newKey=$(sudo kubeadm init phase upload-certs --upload-certs | sed -n '3~0p')

# create vip.yaml in manifests of new control plane
# echo "$(ip route get 8.8.8.8 | awk -F"dev " 'NR==1{split($2,a," ");print a[1]}')"

# was working here
echo "$vipCMD && sudo $(kubeadm token create --print-join-command) --control-plane --certificate-key $newKey" | xclip -sel clip
echo "..."
echo "Done starting cluster, control-plane join command is copied, don't forget to move load-balancer virtual ip yaml over to new nodes"

exit 0
